def print_rate_progress(n, s, step=0.02, message=''):
    """
    打印进度条及百分比。

    参数：
    n (int): 当前进度。
    s (int): 总进度。
    step (float, optional): 进度条步长，默认为0.02。
    message (str, optional): 附加的消息，默认为空。

    返回：
    无

    """
    if step >= 1:
        step = 0.1
    r = n / s
    t = int(r / step)
    print("\r%s%s %.2f%% %s" % ('-' * t, '*' * (50 - t), r * 100, message), end="")


def set_date_index(df, key):
    """
    将DataFrame的某一列转换为日期索引。

    参数：
    df (pandas.DataFrame): 需要操作的DataFrame。
    key (str): 日期列的名称。

    返回：
    pandas.DataFrame: 转换后的DataFrame。

    """
    import pandas as pd
    data = [pd.Timestamp(i) for i in df[key]]
    del df[key]
    idx = pd.DatetimeIndex(data)
    df.set_index(idx, inplace=True)
    return df


def digit(obj):
    """
    将数字格式化为可读性更好的字符串表示。

    参数：
    obj (int or float): 需要格式化的数字。

    返回：
    tuple: 格式化后的字符串和数字的位数。

    """
    def _f(x):
        if x < 1000:
            return x

        d = ('', 'K', 'M', 'B', 'T')
        for i in range(1, 5):
            x /= 1000
            if x < 1:
                return "%.2f%s" % (x * 1000, d[i - 1])
        return "%.2f%s" % (x, d[-1])

    return len(str(obj)), _f(obj)


def count(x):
    """
    计算列表的长度，并将长度格式化为可读性更好的字符串表示。

    参数：
    x (list): 需要计算长度的列表。

    返回：
    str: 格式化后的字符串。

    """
    length = len(x)
    return digit(length)[1]


def value(s):
    """
    从字符串中提取数字。

    参数：
    s (str): 需要提取数字的字符串。

    返回：
    list: 提取出的数字列表。

    """
    if type(s) != str:
        return [0]
    import re
    t = re.findall(r"\d+\.?\d*", s)
    return [float(i) for i in t]


def count2int(x):
    """
    将带有单位的数字字符串转换为对应的整数。

    参数：
    x (int or str): 需要转换的数字或字符串。

    返回：
    int: 转换后的整数。

    """
    _d = {'K': 1000, 'M': 1000000, 'B': 1000000000, 'T': 1000000000000}
    if isinstance(x, str):
        if x[-1] in _d.keys():
            x = float(x[:-1]) * _d[x[-1]]
        else:
            return 0
    return x



__title__ = 'ANBUtils'
__version__ = '1.5.5'
__author__ = 'redbson'
__email__ = 'redbson@gmail.com'

from .environ_cheker import environment_checker as _checker
_checker()


from .a import (
    print_rate_progress, set_date_index, digit, count, value, count2int
)

from .db_worker import (
    DBWorker, crawler_starter, log_db, read_log, dblink, dblink_add, dblink_remove, dblink_update, collection_show,
    df2mongo, mongo2df, get_db_info, get_mongodb, dblink_help, get_token,
    in_severs
)

from .easy_pickle import (
    easy_dump, easy_load
)

from .id_work import (
    int_mark, id_analyst, matplot_set
)

from .messager import (
    dingtalk_message, dingtalk_text_message, message_mark_A, message_mark_B, message_mark_C, message_mark_D
)

from .tbox import (
    future, future_base, date_format, today, yesterday, tomorrow, ts2str, now, utc2tz
)



# -*- coding: utf-8 -*-

import math

import matplotlib.pyplot as plt
import numpy as np


def matplot_set():
    """
    设置Matplotlib的绘图参数。

    通过调用此函数，可以设置Matplotlib的绘图参数，包括图形大小等。

    参数：
        无

    返回：
        无
    """
    import matplotlib as mplt
    mplt.rcParams['figure.figsize'] = (20, 8)


def int_mark(statr: int, end: int, step: int, extract: int,
             isReplace: bool = False, isShuffle: bool = False) -> list:
    """
    生成整数标记列表。

    根据给定的起始值、结束值、步长和抽取数量，生成一个整数标记列表。可以选择是否进行替换和洗牌。

    参数：
        statr (int): 起始值。
        end (int): 结束值。
        step (int): 步长。
        extract (int): 抽取数量。
        isReplace (bool, 可选): 是否进行放回抽样，默认为False。
        isShuffle (bool, 可选): 是否进行洗牌，默认为False。

    返回：
        list: 生成的整数标记列表。

    异常：
        ValueError: 当起始值、结束值、步长和抽取数量不是整数类型时引发异常。
        ValueError: 当 [ start + step ] 的值小于等于 [ end ] 时引发异常。
        ValueError: 当模式 isReplace 为 False 时，步长必须大于抽取数量。
    """
    if not type(statr) == type(end) == type(step) == type(extract) == int:
        raise ValueError(' start | end | step | extract 的类型必须为 int!')

    if statr + step > end:
        raise ValueError(' [ start + step ] 必须大于 [ end ]!')

    if not isReplace and extract >= step:
        raise ValueError(' 当模式 isReplace = False 时，步长必须大于抽取数量!')

    S = math.floor(statr / step)
    E = math.ceil(end / step)

    a = list(range(step))

    ids = []

    for i in range(S, E):
        at = np.random.choice(a, size=extract, replace=isReplace)
        for t in at:
            ids.append(i * step + t)

    if isShuffle:
        np.random.shuffle(ids)

    return ids


def id_analyst(data, steps=10, plot='pyplot'):
    """
    分析标记数据。

    根据给定的标记数据，进行分析并返回分析结果。

    参数：
        data: 标记数据。
        steps (int, 可选): 分析的步数，默认为10。
        plot (str, 可选): 绘图方式，默认为'pyplot'。

    返回：
        dict: 分析结果。

    异常：
        ImportError: 当无法导入Matlab相关模块时引发异常。
    """
    from math import log10, ceil
    d = sorted(data)
    nd_max = int(log10(d[-1]))
    nd_min = int(log10(d[0]))

    if nd_max != nd_min:
        start = 0
    else:
        start = int(d[0] / (10 ** (nd_min - 1))) * (10 ** (nd_min - 1))

    end = ceil(d[-1] / (10 ** (nd_max - 1))) * (10 ** (nd_max - 1))
    step = (end - start) / steps
    counter = [0] * (steps + 1)

    for i in d:
        counter[int((i - start) / step)] += 1

    rtn = {
        "counter": counter,
        "step": step,
        "start": start,
        "end": end
    }

    x = [i * step + start for i in range(steps + 1)]
    y = counter

    if plot == "pyplot":
        matplot_set()
        plt.plot(x, y)
        plt.show()

    elif plot == "matlab":
        import matlab
        import matlab.engine
        eng = matlab.engine.start_matlab()
        X = matlab.double(x)
        Y = matlab.double(y)
        eng.plot(X, Y)
        rtn['plot'] = eng

    return rtn



# -*- coding: utf-8 -*-

import os
import requests

# 定义消息标记常量
message_mark_0: str = 'Message'
message_mark_A: str = '[脑暴]'
message_mark_B: str = '[赞]'
message_mark_C: str = '[加油]'
message_mark_D: str = '[算账]'

msg_text = '''{
    "msgtype": "text",
    "text": {
        "content": "%s"
    }
}'''


def dingtalk_message(data):
    """
    发送钉钉消息。

    根据给定的数据，通过钉钉机器人的Webhook发送消息。

    参数：
        data: 消息数据。

    返回：
        str: 发送结果。

    异常：
        None
    """
    webhook = os.environ.get('DINGTALK_WEBHOOK')
    headers = {
        'Content-Type': 'application/json',
    }
    response = requests.post(webhook, headers=headers, data=data.encode("UTF-8"))
    return response.text


def dingtalk_text_message(text: str):
    """
    发送钉钉文本消息。

    根据给定的文本内容，通过钉钉机器人发送文本消息。

    参数：
        text (str): 文本内容。

    返回：
        None

    异常：
        None
    """
    dingtalk_message(msg_text % text)



from setuptools import setup, find_packages

setup(
    name="ANBUtils",
    version="1.5.5",
    packages=find_packages(),
    author="redbson",
    author_email="redbson@gmail.com",
    python_requires='>=3.8',
    install_requires=[
        "matplotlib>=3.0.0",
        "numpy>=1.0.0",
        "pandas>=1.0.0",
        "pymongo>=4.0.2",
        "requests>=2.0.0",
        "matplotlib>=3.0.0",

    ]
)
